#! /usr/bin/perl

require 5.004;

# -------------------------------------------------------------------[ Uses ]--

use Socket;
use strict;
use Getopt::Long;
use vars qw($opt_V $opt_h $opt_t $opt_p $opt_H $opt_s $verbose);
use vars qw($PROGNAME);
use lib "/usr/local/nagios/nagmain/libexec" ;

# ----------------------------------------------------[ Function Prototypes ]--

sub print_help ();
sub print_usage ();
sub bindRemote ($$$);
sub usage ($);

# -------------------------------------------------------------[ Enviroment ]--

$ENV{PATH} = "";
$ENV{ENV} = "";
$ENV{BASH_ENV} = "";

# -----------------------------------------------------------------[ Global ]--

$PROGNAME = "check_munin";
my $timeout=30;
	
# -------------------------------------------------------------[ usage ]--
sub usage ($)
{
	my ($errstr) = @_;

	print "Error: $errstr \n";
	print_usage();
	exit 1;
}

# ------------------------------------------------------------[ print_usage ]--

sub print_usage () {
	print "Usage: $PROGNAME -H <host> -s <munin_plugin> [-p <port>]\n";
}

# -------------------------------------------------------------[ print_help ]--

sub print_help ()
{
	print "Insert help junk here
Blah Blah Blah

";
	print_usage();
	print "
-H, --hostname=HOST
   Name or IP address of host to check
-s, --script=PLUGIN_NAME
   Name of the Munin plugin to query
-p, --port=INTEGER
   Port that the munin daemon is running on <host> (Default: 4949)
-v, --verbose
   Print extra debugging information
";
}

# -------------------------------------------------------------[ bindRemote ]--

sub bindRemote ($$$)
{
	my ($in_remotehost, $in_remoteport, $in_hostname) = @_;
	my $proto = getprotobyname('tcp');
	my $sockaddr;
	my $this;
	my $thisaddr = gethostbyname($in_hostname);
	my $that;
	my ($name, $aliases,$type,$len,$thataddr) = gethostbyname($in_remotehost);
#	($name,$aliases,$type,$len,$thisaddr) = gethostbyname($in_hostname);

	if (!socket(ClientSocket,AF_INET, SOCK_STREAM, $proto)) {
	    print "MUNIN NODE UNKNOWN: Could not start socket ($!)\n";
	    exit 2;
	}
	$sockaddr = 'S n a4 x8';
	$this = pack($sockaddr, AF_INET, 0, $thisaddr);
	$that = pack($sockaddr, AF_INET, $in_remoteport, $thataddr);
	if (!bind(ClientSocket, $this)) {
	    print "MUNIN NODE UNKNOWN: Could not bind socket ($!)\n";
	    exit 3;
	}
	if (!connect(ClientSocket, $that)) { 
	    print "MUNIN NODE UNKNOWN: Could not connect socket ($!)\n";
	    exit 4;
	}
	select(ClientSocket); $| = 1; select(STDOUT);
	return \*ClientSocket;
}

# ===================================================================[ MAIN ]==

MAIN:
{
	my $hostname;
	my $successValue;
	my $referralValue;
	my $nxrrsetValue;
	my $nxdomainValue;
	my $recursionValue;
	my $failureValue;
	my @row;

	Getopt::Long::Configure('bundling');
	GetOptions
	 ("V"   => \$opt_V,  "version"    => \$opt_V,
		"h"   => \$opt_h,  "help"       => \$opt_h,
		"v"   => \$verbose,"verbose"    => \$verbose,
		"t=i" => \$opt_t,  "timeout=i"  => \$opt_t,
		"s=s" => \$opt_s,  "script=s"  => \$opt_s,
		"p=i" => \$opt_p,  "port=i"     => \$opt_p,
		"H=s" => \$opt_H,  "hostname=s" => \$opt_H);

	if ($opt_V) {
		print "Some bogus revision number\n";
		exit 0;
	}

	if ($opt_h) {print_help(); exit 0;}

	($opt_H) || ($opt_H = shift) || usage("Host name/address not specified\n");
	# deal with hostnames like "TCP:hostname"
	if ($opt_H =~ /^[a-zA-Z]+:(.+)/) {
		$opt_H = $1;
	}
	my $remotehost = $1 if ($opt_H =~ /([-.A-Za-z0-9]+)/);
	($remotehost) || usage("Invalid host: $opt_H\n");

	# deal with hostnames like "TCP:hostname"
	if ($remotehost =~ /^[a-zA-Z]:(.+)/) {
		$remotehost = $1;
print "hostname is $remotehost\n";
	}

	#if ($opt_s) {print_help(); exit 0;}
	if (length($opt_s) == 0) {print_help(); exit 0;}
	my $pluginName = $opt_s;

	($opt_p) || ($opt_p = shift) || ($opt_p = 4949);
	my $remoteport = $1 if ($opt_p =~ /^([0-9]+)$/);
	($remoteport) || usage("Invalid port: $opt_p\n");

	if ($opt_t && $opt_t =~ /^([0-9]+)$/) { $timeout = $1; }

	# Just in case of problems, let's not wait forever
	$SIG{'ALRM'} = sub {
		print "Somthing is Taking a Long Time, Increase Your timeout (Currently Set At $timeout Seconds)\n";
		exit 5;
	};
	
	alarm($timeout);

	chomp($hostname = `/bin/hostname`);
	$hostname = $1 if ($hostname =~ /([-.a-zA-Z0-9]+)/);
	my ($name, $alias, $proto) = getprotobyname('tcp');
	print "MAIN(debug): hostname = $hostname\n" if $verbose;

	print "MAIN(debug): binding to remote host: $remotehost -> $remoteport -> $hostname\n" if $verbose;
	my $ClientSocket = &bindRemote($remotehost,$remoteport,$hostname);
	
	print ClientSocket "fetch $pluginName\nquit\n";
	
	while (<ClientSocket>) {
		print "MAIN(debug): default var = $_\n" if $verbose;

		alarm(0);

		last if (/^\./);
		next if (/^#/);
	
		@row = split;
		print "$row[0]:$row[1] ";
	}
}
